package com.energy.manage.service.service.autoanalysisconfig.impl;

import com.energy.manage.common.enums.OptionEnums;
import com.energy.manage.common.po.autoanalysisconfigdetail.AutoAnalysisConfigDetailPo;
import com.energy.manage.common.po.autoanalysisconfigdetailoptionrecords.AutoAnalysisConfigDetailOptionRecordsPo;
import com.energy.manage.common.reponse.ResultResp;
import com.energy.manage.service.domain.dto.analysis.AutoAnalysisConfigDetailDto;
import com.energy.manage.service.domain.vo.analysis.AutoAnalysisConfigDetailVo;
import com.energy.manage.service.mappers.autoanalysisconfigdetail.AutoAnalysisConfigDetailMapper;
import com.energy.manage.service.mappers.autoanalysisconfigdetailoptionrecords.AutoAnalysisConfigDetailOptionRecordsMapper;
import com.energy.manage.service.service.autoanalysisconfig.AutoAnalysisConfigService;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

import static com.energy.manage.common.reponse.ConstVar.EXSIT_ON_CONFIG;

/**
 * @author chy
 * @date 2025/3/27 09:50
 * @desc
 */
@Service
public class AutoAnalysisConfigServiceImpl implements AutoAnalysisConfigService {

    @Autowired
    private AutoAnalysisConfigDetailMapper autoAnalysisConfigDetailMapper;

    @Autowired
    private AutoAnalysisConfigDetailOptionRecordsMapper optionRecordsMapper;

    /**
     * @param autoAnalysisConfigDetailDto
     * @return
     */
    @Override
    public List<AutoAnalysisConfigDetailVo> configList(AutoAnalysisConfigDetailDto autoAnalysisConfigDetailDto) {
        return autoAnalysisConfigDetailMapper.selectByCondition(autoAnalysisConfigDetailDto);
    }

    @Override
    public int configSave(AutoAnalysisConfigDetailDto autoAnalysisConfigDetailDto) {
        List<AutoAnalysisConfigDetailVo> vos = autoAnalysisConfigDetailMapper.selectByCondition(autoAnalysisConfigDetailDto);
        if(CollectionUtils.isNotEmpty(vos)){
            return 0;
        }
        //保存配置
        int saveCount = autoAnalysisConfigDetailMapper.insertConfig(autoAnalysisConfigDetailDto);
        //保存操作记录
        optionRecordsMapper.insertSelective(new AutoAnalysisConfigDetailOptionRecordsPo().setFieldCode(autoAnalysisConfigDetailDto.getFieldCode()));
        return saveCount;
    }

    @Override
    public int configEdit(AutoAnalysisConfigDetailDto detailDto) {
        //保存操作记录(先保存原始值)
        saveOptionRecords(detailDto.getId(), OptionEnums.EDIT.getCode());
        //更新配置
        return autoAnalysisConfigDetailMapper.updateConfigByCondition(detailDto);
    }

    /**
     * 启动/关闭
     * @param detailDto
     * @return
     */
    @Override
    public ResultResp onOff(AutoAnalysisConfigDetailDto detailDto) {
        int startupState = detailDto.getStartupState();
        //判断当前风场有无启动中的配置
        if(hasStartupConfig(detailDto)){
            return ResultResp.FAIL(EXSIT_ON_CONFIG);
        }
        //保存操作记录
        int optionFlag = startupState == 1 ? OptionEnums.ON.getCode() : OptionEnums.OFF.getCode();
        //保存操作记录(先保存操作记录，后更新)
        saveOptionRecords(detailDto.getId(), optionFlag);
        //更新配置
        autoAnalysisConfigDetailMapper.updateConfigByCondition(detailDto);
        return ResultResp.SUCCESS();
    }

    /**
     * 是否存在启动中的配置
     * @param detailDto
     * @return
     */
    private Boolean hasStartupConfig(AutoAnalysisConfigDetailDto detailDto){
        if(detailDto.getStartupState() == 0){
            return false;
        }
        //获取当前风场启动中的配置列表
        List<AutoAnalysisConfigDetailVo> vos = autoAnalysisConfigDetailMapper.selectByCondition(new AutoAnalysisConfigDetailDto()
                                                                                                        .setFieldCode(detailDto.getFieldCode()));
        return CollectionUtils.isNotEmpty(vos);
    }

    /**
     * 保存操作记录
     * @param id
     * @param optionFlag
     */
    private void saveOptionRecords(Long id, Integer optionFlag){
        //查询更新前的配置，为后续保存操作记录做准备
        AutoAnalysisConfigDetailPo exist = autoAnalysisConfigDetailMapper.selectByPrimaryKey(id);
        //保存操作记录
        AutoAnalysisConfigDetailOptionRecordsPo recordsPo = new AutoAnalysisConfigDetailOptionRecordsPo()
                                                                    .setFieldCode(exist.getFieldCode())
                                                                    .setOptionFlag(optionFlag)
                                                                    .setSourceStartupState(exist.getStartupState())
                                                                    .setSourceIntevalDays(exist.getIntervalDays())
                                                                    .setSourceStartTime(exist.getStartTime());
        optionRecordsMapper.insertSelective(recordsPo);
    }
}
